@@ -15,13 +15,13 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Any, Callable, Dict, Optional
-from urllib.parse import quote, urlparse
+import urllib.parse
+from typing import Any, Callable, Dict, Optional, Union
 
 import requests
 
 
-class _StdoutStream(object):
+class _StdoutStream:
     def __call__(self, chunk: Any) -> None:
         print(chunk)
 
@@ -51,19 +51,37 @@ def copy_dict(dest: Dict[str, Any], src: Dict[str, Any]) -> None:
             # custom_attributes: {'foo', 'bar'} =>
             #   "custom_attributes['foo']": "bar"
             for dict_k, dict_v in v.items():
-                dest["%s[%s]" % (k, dict_k)] = dict_v
+                dest[f"{k}[{dict_k}]"] = dict_v
         else:
             dest[k] = v
 
 
-def clean_str_id(id: str) -> str:
-    return quote(id, safe="")
+class EncodedId(str):
+    """A custom `str` class that will return the URL-encoded value of the string.
 
+      * Using it recursively will only url-encode the value once.
+      * Can accept either `str` or `int` as input value.
+      * Can be used in an f-string and output the URL-encoded string.
 
-def sanitized_url(url: str) -> str:
-    parsed = urlparse(url)
-    new_path = parsed.path.replace(".", "%2E")
-    return parsed._replace(path=new_path).geturl()
+    Reference to documentation on why this is necessary.
+
+    See::
+
+        https://docs.gitlab.com/ee/api/index.html#namespaced-path-encoding
+        https://docs.gitlab.com/ee/api/index.html#path-parameters
+    """
+
+    # mypy complains if return type other than the class type. So we ignore issue.
+    def __new__(  # type: ignore
+        cls, value: Union[str, int, "EncodedId"]
+    ) -> Union[int, "EncodedId"]:
+        if isinstance(value, (int, EncodedId)):
+            return value
+
+        if not isinstance(value, str):
+            raise TypeError(f"Unsupported type received: {type(value)}")
+        value = urllib.parse.quote(value, safe="")
+        return super().__new__(cls, value)
 
 
 def remove_none_from_dict(data: Dict[str, Any]) -> Dict[str, Any]:
