@@ -27,7 +27,7 @@ import gitlab.v4.objects
 from gitlab import cli
 
 
-class GitlabCLI(object):
+class GitlabCLI:
     def __init__(
         self, gl: gitlab.Gitlab, what: str, action: str, args: Dict[str, str]
     ) -> None:
@@ -39,6 +39,7 @@ class GitlabCLI(object):
         self.action = action.lower()
         self.gl = gl
         self.args = args
+        self.parent_args: Dict[str, Any] = {}
         self.mgr_cls: Union[
             Type[gitlab.mixins.CreateMixin],
             Type[gitlab.mixins.DeleteMixin],
@@ -46,14 +47,17 @@ class GitlabCLI(object):
             Type[gitlab.mixins.GetWithoutIdMixin],
             Type[gitlab.mixins.ListMixin],
             Type[gitlab.mixins.UpdateMixin],
-        ] = getattr(gitlab.v4.objects, self.cls.__name__ + "Manager")
+        ] = getattr(gitlab.v4.objects, f"{self.cls.__name__}Manager")
         # We could do something smart, like splitting the manager name to find
         # parents, build the chain of managers to get to the final object.
         # Instead we do something ugly and efficient: interpolate variables in
         # the class _path attribute, and replace the value with the result.
         if TYPE_CHECKING:
             assert self.mgr_cls._path is not None
-        self.mgr_cls._path = self.mgr_cls._path % self.args
+
+        self._process_from_parent_attrs()
+
+        self.mgr_cls._path = self.mgr_cls._path.format(**self.parent_args)
         self.mgr = self.mgr_cls(gl)
 
         if self.mgr_cls._types:
@@ -63,14 +67,26 @@ class GitlabCLI(object):
                     obj.set_from_cli(self.args[attr_name])
                     self.args[attr_name] = obj.get()
 
+    def _process_from_parent_attrs(self) -> None:
+        """Items in the path need to be url-encoded. There is a 1:1 mapping from
+        mgr_cls._from_parent_attrs <--> mgr_cls._path. Those values must be url-encoded
+        as they may contain a slash '/'."""
+        for key in self.mgr_cls._from_parent_attrs:
+            if key not in self.args:
+                continue
+
+            self.parent_args[key] = gitlab.utils.EncodedId(self.args[key])
+            # If we don't delete it then it will be added to the URL as a query-string
+            del self.args[key]
+
     def __call__(self) -> Any:
         # Check for a method that matches object + action
-        method = "do_%s_%s" % (self.what, self.action)
+        method = f"do_{self.what}_{self.action}"
         if hasattr(self, method):
             return getattr(self, method)()
 
         # Fallback to standard actions (get, list, create, ...)
-        method = "do_%s" % self.action
+        method = f"do_{self.action}"
         if hasattr(self, method):
             return getattr(self, method)()
 
@@ -78,6 +94,7 @@ class GitlabCLI(object):
         return self.do_custom()
 
     def do_custom(self) -> Any:
+        class_instance: Union[gitlab.base.RESTManager, gitlab.base.RESTObject]
         in_obj = cli.custom_actions[self.cls_name][self.action][2]
 
         # Get the object (lazy), then act
@@ -85,21 +102,27 @@ class GitlabCLI(object):
             data = {}
             if self.mgr._from_parent_attrs:
                 for k in self.mgr._from_parent_attrs:
-                    data[k] = self.args[k]
+                    data[k] = self.parent_args[k]
             if not issubclass(self.cls, gitlab.mixins.GetWithoutIdMixin):
                 if TYPE_CHECKING:
                     assert isinstance(self.cls._id_attr, str)
                 data[self.cls._id_attr] = self.args.pop(self.cls._id_attr)
-            obj = self.cls(self.mgr, data)
-            method_name = self.action.replace("-", "_")
-            return getattr(obj, method_name)(**self.args)
+            class_instance = self.cls(self.mgr, data)
         else:
-            return getattr(self.mgr, self.action)(**self.args)
+            class_instance = self.mgr
+
+        method_name = self.action.replace("-", "_")
+        return getattr(class_instance, method_name)(**self.args)
 
     def do_project_export_download(self) -> None:
         try:
             project = self.gl.projects.get(int(self.args["project_id"]), lazy=True)
-            data = project.exports.get().download()
+            export_status = project.exports.get()
+            if TYPE_CHECKING:
+                assert export_status is not None
+            data = export_status.download()
+            if TYPE_CHECKING:
+                assert data is not None
             sys.stdout.buffer.write(data)
 
         except Exception as e:
@@ -174,7 +197,7 @@ class GitlabCLI(object):
 def _populate_sub_parser_by_class(
     cls: Type[gitlab.base.RESTObject], sub_parser: argparse._SubParsersAction
 ) -> None:
-    mgr_cls_name = cls.__name__ + "Manager"
+    mgr_cls_name = f"{cls.__name__}Manager"
     mgr_cls = getattr(gitlab.v4.objects, mgr_cls_name)
 
     for action_name in ["list", "get", "create", "update", "delete"]:
@@ -186,60 +209,60 @@ def _populate_sub_parser_by_class(
         if mgr_cls._from_parent_attrs:
             for x in mgr_cls._from_parent_attrs:
                 sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=True
+                    f"--{x.replace('_', '-')}", required=True
                 )
 
         if action_name == "list":
             for x in mgr_cls._list_filters:
                 sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=False
+                    f"--{x.replace('_', '-')}", required=False
                 )
 
-            sub_parser_action.add_argument("--page", required=False)
-            sub_parser_action.add_argument("--per-page", required=False)
+            sub_parser_action.add_argument("--page", required=False, type=int)
+            sub_parser_action.add_argument("--per-page", required=False, type=int)
             sub_parser_action.add_argument("--all", required=False, action="store_true")
 
         if action_name == "delete":
             if cls._id_attr is not None:
                 id_attr = cls._id_attr.replace("_", "-")
-                sub_parser_action.add_argument("--%s" % id_attr, required=True)
+                sub_parser_action.add_argument(f"--{id_attr}", required=True)
 
         if action_name == "get":
             if not issubclass(cls, gitlab.mixins.GetWithoutIdMixin):
                 if cls._id_attr is not None:
                     id_attr = cls._id_attr.replace("_", "-")
-                    sub_parser_action.add_argument("--%s" % id_attr, required=True)
+                    sub_parser_action.add_argument(f"--{id_attr}", required=True)
 
             for x in mgr_cls._optional_get_attrs:
                 sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=False
+                    f"--{x.replace('_', '-')}", required=False
                 )
 
         if action_name == "create":
             for x in mgr_cls._create_attrs.required:
                 sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=True
+                    f"--{x.replace('_', '-')}", required=True
                 )
             for x in mgr_cls._create_attrs.optional:
                 sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=False
+                    f"--{x.replace('_', '-')}", required=False
                 )
 
         if action_name == "update":
             if cls._id_attr is not None:
                 id_attr = cls._id_attr.replace("_", "-")
-                sub_parser_action.add_argument("--%s" % id_attr, required=True)
+                sub_parser_action.add_argument(f"--{id_attr}", required=True)
 
             for x in mgr_cls._update_attrs.required:
                 if x != cls._id_attr:
                     sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=True
+                        f"--{x.replace('_', '-')}", required=True
                     )
 
             for x in mgr_cls._update_attrs.optional:
                 if x != cls._id_attr:
                     sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=False
+                        f"--{x.replace('_', '-')}", required=False
                     )
 
     if cls.__name__ in cli.custom_actions:
@@ -250,7 +273,7 @@ def _populate_sub_parser_by_class(
             if mgr_cls._from_parent_attrs:
                 for x in mgr_cls._from_parent_attrs:
                     sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=True
+                        f"--{x.replace('_', '-')}", required=True
                     )
                 sub_parser_action.add_argument("--sudo", required=False)
 
@@ -258,23 +281,19 @@ def _populate_sub_parser_by_class(
             if not issubclass(cls, gitlab.mixins.GetWithoutIdMixin):
                 if cls._id_attr is not None:
                     id_attr = cls._id_attr.replace("_", "-")
-                    sub_parser_action.add_argument("--%s" % id_attr, required=True)
+                    sub_parser_action.add_argument(f"--{id_attr}", required=True)
 
             required, optional, dummy = cli.custom_actions[name][action_name]
-            [
-                sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=True
-                )
-                for x in required
-                if x != cls._id_attr
-            ]
-            [
-                sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=False
-                )
-                for x in optional
-                if x != cls._id_attr
-            ]
+            for x in required:
+                if x != cls._id_attr:
+                    sub_parser_action.add_argument(
+                        f"--{x.replace('_', '-')}", required=True
+                    )
+            for x in optional:
+                if x != cls._id_attr:
+                    sub_parser_action.add_argument(
+                        f"--{x.replace('_', '-')}", required=False
+                    )
 
     if mgr_cls.__name__ in cli.custom_actions:
         name = mgr_cls.__name__
@@ -283,25 +302,21 @@ def _populate_sub_parser_by_class(
             if mgr_cls._from_parent_attrs:
                 for x in mgr_cls._from_parent_attrs:
                     sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=True
+                        f"--{x.replace('_', '-')}", required=True
                     )
                 sub_parser_action.add_argument("--sudo", required=False)
 
             required, optional, dummy = cli.custom_actions[name][action_name]
-            [
-                sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=True
-                )
-                for x in required
-                if x != cls._id_attr
-            ]
-            [
-                sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=False
-                )
-                for x in optional
-                if x != cls._id_attr
-            ]
+            for x in required:
+                if x != cls._id_attr:
+                    sub_parser_action.add_argument(
+                        f"--{x.replace('_', '-')}", required=True
+                    )
+            for x in optional:
+                if x != cls._id_attr:
+                    sub_parser_action.add_argument(
+                        f"--{x.replace('_', '-')}", required=False
+                    )
 
 
 def extend_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
@@ -344,7 +359,7 @@ def get_dict(
     return obj.attributes
 
 
-class JSONPrinter(object):
+class JSONPrinter:
     def display(self, d: Union[str, Dict[str, Any]], **kwargs: Any) -> None:
         import json  # noqa
 
@@ -354,14 +369,14 @@ class JSONPrinter(object):
         self,
         data: List[Union[str, gitlab.base.RESTObject]],
         fields: List[str],
-        **kwargs: Any
+        **kwargs: Any,
     ) -> None:
         import json  # noqa
 
         print(json.dumps([get_dict(obj, fields) for obj in data]))
 
 
-class YAMLPrinter(object):
+class YAMLPrinter:
     def display(self, d: Union[str, Dict[str, Any]], **kwargs: Any) -> None:
         try:
             import yaml  # noqa
@@ -378,7 +393,7 @@ class YAMLPrinter(object):
         self,
         data: List[Union[str, gitlab.base.RESTObject]],
         fields: List[str],
-        **kwargs: Any
+        **kwargs: Any,
     ) -> None:
         try:
             import yaml  # noqa
@@ -396,7 +411,7 @@ class YAMLPrinter(object):
             )
 
 
-class LegacyPrinter(object):
+class LegacyPrinter:
     def display(self, d: Union[str, Dict[str, Any]], **kwargs: Any) -> None:
         verbose = kwargs.get("verbose", False)
         padding = kwargs.get("padding", 0)
@@ -408,11 +423,11 @@ class LegacyPrinter(object):
             for k in sorted(d.keys()):
                 v = d[k]
                 if isinstance(v, dict):
-                    print("%s%s:" % (" " * padding, k.replace("_", "-")))
+                    print(f"{' ' * padding}{k.replace('_', '-')}:")
                     new_padding = padding + 2
                     self.display(v, verbose=True, padding=new_padding, obj=v)
                     continue
-                print("%s%s: %s" % (" " * padding, k.replace("_", "-"), v))
+                print(f"{' ' * padding}{k.replace('_', '-')}: {v}")
 
         if verbose:
             if isinstance(obj, dict):
@@ -422,7 +437,7 @@ class LegacyPrinter(object):
             # not a dict, we assume it's a RESTObject
             if obj._id_attr:
                 id = getattr(obj, obj._id_attr, None)
-                print("%s: %s" % (obj._id_attr, id))
+                print(f"{obj._id_attr}: {id}")
             attrs = obj.attributes
             if obj._id_attr:
                 attrs.pop(obj._id_attr)
@@ -433,23 +448,23 @@ class LegacyPrinter(object):
                 assert isinstance(obj, gitlab.base.RESTObject)
             if obj._id_attr:
                 id = getattr(obj, obj._id_attr)
-                print("%s: %s" % (obj._id_attr.replace("_", "-"), id))
+                print(f"{obj._id_attr.replace('_', '-')}: {id}")
             if obj._short_print_attr:
                 value = getattr(obj, obj._short_print_attr) or "None"
                 value = value.replace("\r", "").replace("\n", " ")
                 # If the attribute is a note (ProjectCommitComment) then we do
                 # some modifications to fit everything on one line
-                line = "%s: %s" % (obj._short_print_attr, value)
+                line = f"{obj._short_print_attr}: {value}"
                 # ellipsize long lines (comments)
                 if len(line) > 79:
-                    line = line[:76] + "..."
+                    line = f"{line[:76]}..."
                 print(line)
 
     def display_list(
         self,
         data: List[Union[str, gitlab.base.RESTObject]],
         fields: List[str],
-        **kwargs: Any
+        **kwargs: Any,
     ) -> None:
         verbose = kwargs.get("verbose", False)
         for obj in data:
