@@ -39,6 +39,7 @@ from .export_import import *
 from .features import *
 from .files import *
 from .geo_nodes import *
+from .group_access_tokens import *
 from .groups import *
 from .hooks import *
 from .issues import *
@@ -49,6 +50,7 @@ from .ldap import *
 from .members import *
 from .merge_request_approvals import *
 from .merge_requests import *
+from .merge_trains import *
 from .milestones import *
 from .namespaces import *
 from .notes import *
@@ -57,9 +59,11 @@ from .packages import *
 from .pages import *
 from .personal_access_tokens import *
 from .pipelines import *
+from .project_access_tokens import *
 from .projects import *
 from .push_rules import *
 from .releases import *
+from .repositories import *
 from .runners import *
 from .services import *
 from .settings import *
@@ -69,18 +73,10 @@ from .statistics import *
 from .tags import *
 from .templates import *
 from .todos import *
+from .topics import *
 from .triggers import *
 from .users import *
 from .variables import *
 from .wikis import *
 
-# TODO: deprecate these in favor of gitlab.const.*
-VISIBILITY_PRIVATE = "private"
-VISIBILITY_INTERNAL = "internal"
-VISIBILITY_PUBLIC = "public"
-
-ACCESS_GUEST = 10
-ACCESS_REPORTER = 20
-ACCESS_DEVELOPER = 30
-ACCESS_MASTER = 40
-ACCESS_OWNER = 50
+__all__ = [name for name in dir() if not name.startswith("_")]
