@@ -1,10 +1,11 @@
+from typing import Any, cast, Union
+
 from gitlab import types
 from gitlab.base import RequiredOptional, RESTManager, RESTObject
 from gitlab.mixins import (
     CRUDMixin,
     DeleteMixin,
     ListMixin,
-    MemberAllMixin,
     ObjectDeleteMixin,
     RetrieveMixin,
     SaveMixin,
@@ -16,9 +17,11 @@ __all__ = [
     "GroupBillableMemberMembership",
     "GroupBillableMemberMembershipManager",
     "GroupMember",
+    "GroupMemberAll",
     "GroupMemberManager",
     "GroupMemberAllManager",
     "ProjectMember",
+    "ProjectMemberAll",
     "ProjectMemberManager",
     "ProjectMemberAllManager",
 ]
@@ -28,8 +31,8 @@ class GroupMember(SaveMixin, ObjectDeleteMixin, RESTObject):
     _short_print_attr = "username"
 
 
-class GroupMemberManager(MemberAllMixin, CRUDMixin, RESTManager):
-    _path = "/groups/%(group_id)s/members"
+class GroupMemberManager(CRUDMixin, RESTManager):
+    _path = "/groups/{group_id}/members"
     _obj_cls = GroupMember
     _from_parent_attrs = {"group_id": "id"}
     _create_attrs = RequiredOptional(
@@ -38,16 +41,22 @@ class GroupMemberManager(MemberAllMixin, CRUDMixin, RESTManager):
     _update_attrs = RequiredOptional(
         required=("access_level",), optional=("expires_at",)
     )
-    _types = {"user_ids": types.ListAttribute}
+    _types = {"user_ids": types.CommaSeparatedListAttribute}
+
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> GroupMember:
+        return cast(GroupMember, super().get(id=id, lazy=lazy, **kwargs))
 
 
 class GroupBillableMember(ObjectDeleteMixin, RESTObject):
     _short_print_attr = "username"
-    _managers = (("memberships", "GroupBillableMemberMembershipManager"),)
+
+    memberships: "GroupBillableMemberMembershipManager"
 
 
 class GroupBillableMemberManager(ListMixin, DeleteMixin, RESTManager):
-    _path = "/groups/%(group_id)s/billable_members"
+    _path = "/groups/{group_id}/billable_members"
     _obj_cls = GroupBillableMember
     _from_parent_attrs = {"group_id": "id"}
     _list_filters = ("search", "sort")
@@ -58,23 +67,32 @@ class GroupBillableMemberMembership(RESTObject):
 
 
 class GroupBillableMemberMembershipManager(ListMixin, RESTManager):
-    _path = "/groups/%(group_id)s/billable_members/%(user_id)s/memberships"
+    _path = "/groups/{group_id}/billable_members/{user_id}/memberships"
     _obj_cls = GroupBillableMemberMembership
     _from_parent_attrs = {"group_id": "group_id", "user_id": "id"}
 
 
+class GroupMemberAll(RESTObject):
+    _short_print_attr = "username"
+
+
 class GroupMemberAllManager(RetrieveMixin, RESTManager):
-    _path = "/groups/%(group_id)s/members/all"
-    _obj_cls = GroupMember
+    _path = "/groups/{group_id}/members/all"
+    _obj_cls = GroupMemberAll
     _from_parent_attrs = {"group_id": "id"}
 
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> GroupMemberAll:
+        return cast(GroupMemberAll, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class ProjectMember(SaveMixin, ObjectDeleteMixin, RESTObject):
     _short_print_attr = "username"
 
 
-class ProjectMemberManager(MemberAllMixin, CRUDMixin, RESTManager):
-    _path = "/projects/%(project_id)s/members"
+class ProjectMemberManager(CRUDMixin, RESTManager):
+    _path = "/projects/{project_id}/members"
     _obj_cls = ProjectMember
     _from_parent_attrs = {"project_id": "id"}
     _create_attrs = RequiredOptional(
@@ -83,10 +101,24 @@ class ProjectMemberManager(MemberAllMixin, CRUDMixin, RESTManager):
     _update_attrs = RequiredOptional(
         required=("access_level",), optional=("expires_at",)
     )
-    _types = {"user_ids": types.ListAttribute}
+    _types = {"user_ids": types.CommaSeparatedListAttribute}
+
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> ProjectMember:
+        return cast(ProjectMember, super().get(id=id, lazy=lazy, **kwargs))
+
+
+class ProjectMemberAll(RESTObject):
+    _short_print_attr = "username"
 
 
 class ProjectMemberAllManager(RetrieveMixin, RESTManager):
-    _path = "/projects/%(project_id)s/members/all"
-    _obj_cls = ProjectMember
+    _path = "/projects/{project_id}/members/all"
+    _obj_cls = ProjectMemberAll
     _from_parent_attrs = {"project_id": "id"}
+
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> ProjectMemberAll:
+        return cast(ProjectMemberAll, super().get(id=id, lazy=lazy, **kwargs))
